@@ -21,6 +21,7 @@ import cvxpy.settings as s
 import cvxpy.utilities as u
 from toolz.itertoolz import unique
 import cvxpy.interface as intf
+from cvxpy.error import SolverError
 import cvxpy.lin_ops.lin_utils as lu
 import cvxpy.lin_ops as lo
 import cvxpy.lin_ops.lin_to_matrix as op2mat
@@ -149,13 +150,57 @@ class Problem(u.Canonical):
 
         for constr in self.constraints:
             canon_constr += constr.canonical_form[1]
-        # Remove redundant constraints.
-        canon_constr = unique(canon_constr,
-                              key=lambda c: c.constr_id)
+
         constr_map = self._filter_constraints(canon_constr)
 
         return (obj, constr_map)
 
+    def _presolve(self, objective, constr_map):
+        """Eliminates unnecessary constraints and short circuits the solver
+        if possible.
+
+        Parameters
+        ----------
+        constr_map : dict
+            A map of constraint type to a list of constraints.
+
+        Returns
+        -------
+        bool
+            Is the problem infeasible?
+        """
+        # Remove redundant constraints.
+        for key, constraints in constr_map.items():
+            uniq_constr = unique(constraints,
+                                 key=lambda c: c.constr_id)
+            constr_map[key] = list(uniq_constr)
+
+        # If there are no constraints, the problem is unbounded
+        # if any of the coefficients are non-zero.
+        # If all the coefficients are zero then return the constant term
+        # and set all variables to 0.
+        if not any(constr_map.values()):
+            pass # TODO
+
+        # Remove constraints with no variables.
+        for key in [s.EQ, s.LEQ]:
+            new_constraints = []
+            for constr in constr_map[key]:
+                vars_ = lu.get_expr_vars(constr.expr)
+                if len(vars_) == 0:
+                    coeff = op2mat.get_constant_coeff(constr.expr)
+                    sign = intf.sign(coeff)
+                    # For equality constraint, coeff must be zero.
+                    # For inequality (i.e. <= 0) constraint, coeff must be negative.
+                    if key is s.EQ and not sign.is_zero() or \
+                       key is s.LEQ and not sign.is_negative():
+                       return s.INFEASIBLE
+                else:
+                    new_constraints.append(constr)
+            constr_map[key] = new_constraints
+
+        return None
+
     def _format_for_solver(self, constr_map, solver):
         """Formats the problem for the solver.
 
@@ -227,7 +272,7 @@ class Problem(u.Canonical):
         if (constr_map[s.SDP] and not solver in s.SDP_CAPABLE) or \
            (constr_map[s.EXP] and not solver in s.EXP_CAPABLE) or \
            (self._constraints_count(constr_map) == 0 and solver == s.SCS):
-            raise Exception(
+            raise SolverError(
                 "The solver %s cannot solve the problem." % solver
             )
 
@@ -305,6 +350,7 @@ class Problem(u.Canonical):
             arguments to solver
         """
         objective, constr_map = self.canonicalize()
+        self._presolve(objective, constr_map)
         # Raise an error if the solver cannot handle the problem.
         self._validate_solver(constr_map, solver)
         dims = self._format_for_solver(constr_map, solver)
@@ -322,11 +368,10 @@ class Problem(u.Canonical):
             args, offset = self._scs_problem_data(objective, constr_map, dims,
                                                   var_offsets, x_length)
         else:
-            raise Exception("Cannot return problem data for the solver %s." % solver)
+            raise SolverError("Cannot return problem data for the solver %s." % solver)
         return args
 
-    def _solve(self, solver=None, ignore_dcp=False, verbose=False,
-               solver_specific_opts=None):
+    def _solve(self, solver=None, ignore_dcp=False, verbose=False, **kwargs):
         """Solves a DCP compliant optimization problem.
 
         Saves the values of primal and dual variables in the variable
@@ -341,7 +386,7 @@ class Problem(u.Canonical):
             DCP.
         verbose : bool, optional
             Overrides the default of hiding solver output.
-        solver_specific_opts : dict, optional
+        kwargs : dict, optional
             A dict of options that will be passed to the specific solver.
             In general, these options will override any default settings
             imposed by cvxpy.
@@ -352,10 +397,6 @@ class Problem(u.Canonical):
             The optimal value for the problem, or a string indicating
             why the problem could not be solved.
         """
-        # Safely set default as empty dict.
-        if solver_specific_opts is None:
-            solver_specific_opts = {}
-
         if not self.is_dcp():
             if ignore_dcp:
                 print ("Problem does not follow DCP rules. "
@@ -364,47 +405,55 @@ class Problem(u.Canonical):
                 raise Exception("Problem does not follow DCP rules.")
 
         objective, constr_map = self.canonicalize()
-        # Choose a default solver if none specified.
-        if solver is None:
-            solver = self._choose_solver(constr_map)
-        else:
-            # Raise an error if the solver cannot handle the problem.
-            self._validate_solver(constr_map, solver)
+        status = self._presolve(objective, constr_map)
+        # The presolver cannot determine the status.
+        if status is None:
+            # Choose a default solver if none specified.
+            if solver is None:
+                solver = self._choose_solver(constr_map)
+            else:
+                # Raise an error if the solver cannot handle the problem.
+                self._validate_solver(constr_map, solver)
 
-        dims = self._format_for_solver(constr_map, solver)
+            dims = self._format_for_solver(constr_map, solver)
 
-        all_ineq = constr_map[s.EQ] + constr_map[s.LEQ]
-        # CVXOPT can have variables that only live in NonLinearConstraints.
-        nonlinear = []
-        if solver == s.CVXOPT:
-            nonlinear = constr_map[s.EXP]
-        var_offsets, var_sizes, x_length = self._get_var_offsets(objective,
-                                                                 all_ineq,
-                                                                 nonlinear)
+            all_ineq = constr_map[s.EQ] + constr_map[s.LEQ]
+            # CVXOPT can have variables that only live in NonLinearConstraints.
+            nonlinear = []
+            if solver == s.CVXOPT:
+                nonlinear = constr_map[s.EXP]
+            var_offsets, var_sizes, x_length = self._get_var_offsets(objective,
+                                                                     all_ineq,
+                                                                     nonlinear)
 
-        if solver == s.CVXOPT:
-            result = self._cvxopt_solve(objective, constr_map, dims,
-                                        var_offsets, x_length,
-                                        verbose, solver_specific_opts)
-        elif solver == s.SCS:
-            result = self._scs_solve(objective, constr_map, dims,
-                                     var_offsets, x_length,
-                                     verbose, solver_specific_opts)
-        elif solver == s.ECOS:
-            result = self._ecos_solve(objective, constr_map, dims,
-                                      var_offsets, x_length,
-                                      verbose, solver_specific_opts)
-        else:
-            raise Exception("Unknown solver.")
+            if solver == s.CVXOPT:
+                result = self._cvxopt_solve(objective, constr_map, dims,
+                                            var_offsets, x_length,
+                                            verbose, kwargs)
+            elif solver == s.SCS:
+                result = self._scs_solve(objective, constr_map, dims,
+                                         var_offsets, x_length,
+                                         verbose, kwargs)
+            elif solver == s.ECOS:
+                result = self._ecos_solve(objective, constr_map, dims,
+                                          var_offsets, x_length,
+                                          verbose, kwargs)
+            else:
+                raise SolverError("Unknown solver.")
 
-        status, value, x, y, z = result
-        if status == s.OPTIMAL:
-            self._save_values(x, self.variables(), var_offsets)
-            self._save_dual_values(y, constr_map[s.EQ], EqConstraint)
-            self._save_dual_values(z, constr_map[s.LEQ], LeqConstraint)
-            self._value = value
-        else:
-            self._handle_failure(status)
+            status, value, x, y, z = result
+            if status in s.SOLUTION_PRESENT:
+                self._save_values(x, self.variables(), var_offsets)
+                self._save_dual_values(y, constr_map[s.EQ], EqConstraint)
+                self._save_dual_values(z, constr_map[s.LEQ], LeqConstraint)
+                self._value = value
+            elif status == s.SOLVER_ERROR:
+                raise SolverError(
+                    "Solver '%s' failed. Try another solver." % solver
+                )
+        # Infeasible or unbounded.
+        if status in s.INF_OR_UNB:
+            self._handle_no_solution(status)
         self._status = status
         return self.value
 
@@ -476,9 +525,9 @@ class Problem(u.Canonical):
         prob_data = self._ecos_problem_data(objective, constr_map, dims,
                                             var_offsets, x_length)
         obj_offset = prob_data[1]
-        results = ecos.solve(*prob_data[0], verbose=verbose)
+        results = ecos.solve(*prob_data[0], verbose=verbose, **opts)
         status = s.SOLVER_STATUS[s.ECOS][results['info']['exitFlag']]
-        if status == s.OPTIMAL:
+        if status in s.SOLUTION_PRESENT:
             primal_val = results['info']['pcost']
             value = self.objective._primal_to_result(
                           primal_val - obj_offset)
@@ -597,7 +646,7 @@ class Problem(u.Canonical):
 
         # Restore original cvxopt solver options.
         cvxopt.solvers.options = old_options
-        if status == s.OPTIMAL:
+        if status in s.SOLUTION_PRESENT:
             primal_val = results['primal objective']
             value = self.objective._primal_to_result(
                           primal_val - obj_offset)
@@ -683,7 +732,7 @@ class Problem(u.Canonical):
         use_indirect = opts["USE_INDIRECT"] if "USE_INDIRECT" in opts else False
         results = scs.solve(*prob_data[0], opts=opts, USE_INDIRECT = use_indirect)
         status = s.SOLVER_STATUS[s.SCS][results["info"]["status"]]
-        if status == s.OPTIMAL:
+        if status in s.SOLUTION_PRESENT:
             primal_val = results["info"]["pobj"]
             value = self.objective._primal_to_result(primal_val - obj_offset)
             eq_dual = results["y"][0:dims["f"]]
@@ -692,8 +741,8 @@ class Problem(u.Canonical):
         else:
             return (status, None, None, None, None)
 
-    def _handle_failure(self, status):
-        """Updates value fields based on the cause of solver failure.
+    def _handle_no_solution(self, status):
+        """Updates value fields when the problem is infeasible or unbounded.
 
         Parameters
         ----------
@@ -710,8 +759,6 @@ class Problem(u.Canonical):
             self._value = self.objective._primal_to_result(np.inf)
         elif status == s.UNBOUNDED:
             self._value = self.objective._primal_to_result(-np.inf)
-        else: # Solver error
-            self._value = None
 
     def _get_var_offsets(self, objective, constraints, nonlinear=None):
         """Maps each variable to a horizontal offset.
