@@ -189,6 +189,7 @@ class TestProblem(BaseTest):
         obj = 0
         def test(self):
             objective, constr_map  = self.canonicalize()
+            self._presolve(objective, constr_map)
             print len(constr_map[s.SOC])
             dims = self._format_for_solver(constr_map, s.ECOS)
             return (len(constr_map[s.EQ]),len(constr_map[s.LEQ]))
@@ -988,14 +989,22 @@ class TestProblem(BaseTest):
         self.assertAlmostEqual(result, -6)
         self.assertItemsAlmostEqual(expr.value, 2*c)
 
-    def test_cvxopt_errors(self):
-        """Tests that cvxopt errors are caught as solver_error.
+    def test_solver_errors(self):
+        """Tests that solver errors throw an exception.
         """
         # For some reason CVXOPT can't handle this problem.
         expr = 500*self.a + square(self.a)
         prob = Problem(Minimize(expr))
-        prob.solve(solver=s.CVXOPT)
-        self.assertEqual(prob.status, s.SOLVER_ERROR)
+
+        with self.assertRaises(Exception) as cm:
+            prob.solve(solver=s.CVXOPT)
+        self.assertEqual(str(cm.exception),
+            "Solver 'CVXOPT' failed. Try another solver.")
+
+        with self.assertRaises(Exception) as cm:
+            Problem(Maximize(sqrt(self.a))).solve(solver=s.ECOS)
+        self.assertEqual(str(cm.exception),
+            "Solver 'ECOS' failed. Try another solver.")
 
     def test_diag_prob(self):
         """Test a problem with diag.
@@ -1009,3 +1018,15 @@ class TestProblem(BaseTest):
         prob = Problem(obj, constraints)
         result = prob.solve()
         self.assertAlmostEqual(result, 0.583151)
+
+    def test_presolve_constant_constraints(self):
+        """Test that the presolver removes constraints with no variables.
+        """
+        x = Variable()
+        obj = Maximize(sqrt(x))
+        prob = Problem(obj)
+        c, G, h, dims, A, b = prob.get_problem_data(s.ECOS)
+        for row in range(A.shape[0]):
+            assert A[row, :].nnz > 0
+        for row in range(G.shape[0]):
+            assert G[row, :].nnz > 0
